#include <xc.h>

#include <stdint.h>
#include <stdbool.h>

#include "lcd.h"

void LCDInit(void)
{
    LCDCONbits.SLPEN=0; // 0 => enable in sleep
//    LCDCONbits.CS=0b10; // 0b1x => LFINTOSC
    LCDCONbits.CS=0b01; // 0b01 => T1OSC
    LCDCONbits.LMUX=0b00; // 0b00 => static
    
    LCDPSbits.WFT=1;
    LCDPSbits.LCDA=1;
//    LCDPSbits.LP=0b1111; // 0b1111 => 1:16 prescaler
//    LCDPSbits.LP=0b0011; // 0b0011 => 1:4 prescaler
    LCDPSbits.LP=0b0000; // 0b0000 => 1:1 prescaler    
    LCDPSbits.BIASMD=0;
    LCDPSbits.WA=1;
    
    LCDREFbits.LCDIRE=0;
    LCDREFbits.LCDIRI=0;
    
    LCDREFbits.VLCD3PE=1;
          
    LCDCSTbits.LCDCST=0b111;
    
    LCDRLbits.LRLAP=0b00;
    LCDRLbits.LRLAT=0b00;
    LCDRLbits.LRLAT=0b000;
    
    LCD_TRIS_COL=1;	LCD_SEGEN_COL=1;

    LCD_TRIS_1ADE=1;LCD_SEGEN_1ADE=1;
    LCD_TRIS_1B=1;	LCD_SEGEN_1B=1;
    LCD_TRIS_1C=1;	LCD_SEGEN_1C=1;
    LCD_TRIS_1F=1;	LCD_SEGEN_1F=1;
    LCD_TRIS_1G=1;	LCD_SEGEN_1G=1;

    LCD_TRIS_2A=1;	LCD_SEGEN_2A=1;
    LCD_TRIS_2B=1;	LCD_SEGEN_2B=1;
    LCD_TRIS_2C=1;	LCD_SEGEN_2C=1;
    LCD_TRIS_2D=1;	LCD_SEGEN_2D=1;
    LCD_TRIS_2E=1;	LCD_SEGEN_2E=1;
    LCD_TRIS_2F=1;	LCD_SEGEN_2F=1;
    LCD_TRIS_2G=1;	LCD_SEGEN_2G=1;

    LCD_TRIS_3A=1;	LCD_SEGEN_3A=1;
    LCD_TRIS_3B=1;	LCD_SEGEN_3B=1;
    LCD_TRIS_3C=1;	LCD_SEGEN_3C=1;
    LCD_TRIS_3D=1;	LCD_SEGEN_3D=1;
    LCD_TRIS_3E=1;	LCD_SEGEN_3E=1;
    LCD_TRIS_3F=1;	LCD_SEGEN_3F=1;
    LCD_TRIS_3G=1;	LCD_SEGEN_3G=1;

    LCD_TRIS_4A=1;	LCD_SEGEN_4A=1;
    LCD_TRIS_4B=1;	LCD_SEGEN_4B=1;
    LCD_TRIS_4C=1;	LCD_SEGEN_4C=1;
    LCD_TRIS_4D=1;	LCD_SEGEN_4D=1;
    LCD_TRIS_4E=1;	LCD_SEGEN_4E=1;
    LCD_TRIS_4F=1;	LCD_SEGEN_4F=1;
    LCD_TRIS_4G=1;	LCD_SEGEN_4G=1;

    // Test pattern, all segments on
    LCD_DATA_COL=1;
    LCD_DATA_1ADE=1;LCD_DATA_1B=1;	LCD_DATA_1C=1;	LCD_DATA_1F=1;	LCD_DATA_1G=1;
    LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=1;	LCD_DATA_2F=1;	LCD_DATA_2G=1;
    LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=1;	LCD_DATA_3F=1;	LCD_DATA_3G=1;
    LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=1;	LCD_DATA_4F=1;	LCD_DATA_4G=1;

    LCDCONbits.LCDEN=1;    
}

// These might look horrid, but we have tons of Flash and this is computationally efficient (i.e., low power) considering
// we have fairly arbitrarily assigned segments, and a bit set is a single instruction. Making it table
// driven might be more beautiful to look at but has a computational cost. Most of this repetitive code was generated by a spreadsheet.
void LCDDigit1Set(uint8_t u8)
{
    switch(u8)
    {
        case 0:
            LCD_DATA_1ADE=1;	LCD_DATA_1B=1;	LCD_DATA_1C=1;	LCD_DATA_1F=1;	LCD_DATA_1G=0;
            break;
        case 1:
            LCD_DATA_1ADE=0;	LCD_DATA_1B=1;	LCD_DATA_1C=1;	LCD_DATA_1F=0;	LCD_DATA_1G=0;
            break;
        case 2:
            LCD_DATA_1ADE=1;	LCD_DATA_1B=1;	LCD_DATA_1C=0;	LCD_DATA_1F=0;	LCD_DATA_1G=1;
            break;
        default:
            LCD_DATA_1ADE=0;	LCD_DATA_1B=0;	LCD_DATA_1C=0;	LCD_DATA_1F=0;	LCD_DATA_1G=0;
            break;
    }
}

void LCDDigit2Set(uint8_t u8)
{
    switch(u8)
    {
        case 0:
            LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=1;	LCD_DATA_2F=1;	LCD_DATA_2G=0;
            break;
        case 1:
            LCD_DATA_2A=0;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=0;	LCD_DATA_2E=0;	LCD_DATA_2F=0;	LCD_DATA_2G=0;
            break;
        case 2:
            LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=0;	LCD_DATA_2D=1;	LCD_DATA_2E=1;	LCD_DATA_2F=0;	LCD_DATA_2G=1;
            break;
        case 3:
            LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=0;	LCD_DATA_2F=0;	LCD_DATA_2G=1;
            break;
        case 4:
            LCD_DATA_2A=0;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=0;	LCD_DATA_2E=0;	LCD_DATA_2F=1;	LCD_DATA_2G=1;
            break;
        case 5:
            LCD_DATA_2A=1;	LCD_DATA_2B=0;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=0;	LCD_DATA_2F=1;	LCD_DATA_2G=1;
            break;
        case 6:
            LCD_DATA_2A=1;	LCD_DATA_2B=0;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=1;	LCD_DATA_2F=1;	LCD_DATA_2G=1;
            break;
        case 7:
            LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=0;	LCD_DATA_2E=0;	LCD_DATA_2F=0;	LCD_DATA_2G=0;
            break;
        case 8:
            LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=1;	LCD_DATA_2F=1;	LCD_DATA_2G=1;
            break;
        case 9:
            LCD_DATA_2A=1;	LCD_DATA_2B=1;	LCD_DATA_2C=1;	LCD_DATA_2D=1;	LCD_DATA_2E=0;	LCD_DATA_2F=1;	LCD_DATA_2G=1;
            break;
        default:
            LCD_DATA_2A=0;	LCD_DATA_2B=0;	LCD_DATA_2C=0;	LCD_DATA_2D=0;	LCD_DATA_2E=0;	LCD_DATA_2F=0;	LCD_DATA_2G=0;
            break;
    }
}
void LCDDigit3Set(uint8_t u8)
{
    switch(u8)
    {
        case 0:
            LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=1;	LCD_DATA_3F=1;	LCD_DATA_3G=0;
            break;
        case 1:
            LCD_DATA_3A=0;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=0;	LCD_DATA_3E=0;	LCD_DATA_3F=0;	LCD_DATA_3G=0;
            break;
        case 2:
            LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=0;	LCD_DATA_3D=1;	LCD_DATA_3E=1;	LCD_DATA_3F=0;	LCD_DATA_3G=1;
            break;
        case 3:
            LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=0;	LCD_DATA_3F=0;	LCD_DATA_3G=1;
            break;
        case 4:
            LCD_DATA_3A=0;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=0;	LCD_DATA_3E=0;	LCD_DATA_3F=1;	LCD_DATA_3G=1;
            break;
        case 5:
            LCD_DATA_3A=1;	LCD_DATA_3B=0;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=0;	LCD_DATA_3F=1;	LCD_DATA_3G=1;
            break;
        case 6:
            LCD_DATA_3A=1;	LCD_DATA_3B=0;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=1;	LCD_DATA_3F=1;	LCD_DATA_3G=1;
            break;
        case 7:
            LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=0;	LCD_DATA_3E=0;	LCD_DATA_3F=0;	LCD_DATA_3G=0;
            break;
        case 8:
            LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=1;	LCD_DATA_3F=1;	LCD_DATA_3G=1;
            break;
        case 9:
            LCD_DATA_3A=1;	LCD_DATA_3B=1;	LCD_DATA_3C=1;	LCD_DATA_3D=1;	LCD_DATA_3E=0;	LCD_DATA_3F=1;	LCD_DATA_3G=1;
            break;
        default:
            LCD_DATA_3A=0;	LCD_DATA_3B=0;	LCD_DATA_3C=0;	LCD_DATA_3D=0;	LCD_DATA_3E=0;	LCD_DATA_3F=0;	LCD_DATA_3G=0;
            break;
    }
}

void LCDDigit4Set(uint8_t u8)
{
    switch(u8)
    {
        case 0:
            LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=1;	LCD_DATA_4F=1;	LCD_DATA_4G=0;
            break;
        case 1:
            LCD_DATA_4A=0;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=0;	LCD_DATA_4E=0;	LCD_DATA_4F=0;	LCD_DATA_4G=0;
            break;
        case 2:
            LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=0;	LCD_DATA_4D=1;	LCD_DATA_4E=1;	LCD_DATA_4F=0;	LCD_DATA_4G=1;
            break;
        case 3:
            LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=0;	LCD_DATA_4F=0;	LCD_DATA_4G=1;
            break;
        case 4:
            LCD_DATA_4A=0;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=0;	LCD_DATA_4E=0;	LCD_DATA_4F=1;	LCD_DATA_4G=1;
            break;
        case 5:
            LCD_DATA_4A=1;	LCD_DATA_4B=0;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=0;	LCD_DATA_4F=1;	LCD_DATA_4G=1;
            break;
        case 6:
            LCD_DATA_4A=1;	LCD_DATA_4B=0;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=1;	LCD_DATA_4F=1;	LCD_DATA_4G=1;
            break;
        case 7:
            LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=0;	LCD_DATA_4E=0;	LCD_DATA_4F=0;	LCD_DATA_4G=0;
            break;
        case 8:
            LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=1;	LCD_DATA_4F=1;	LCD_DATA_4G=1;
            break;
        case 9:
            LCD_DATA_4A=1;	LCD_DATA_4B=1;	LCD_DATA_4C=1;	LCD_DATA_4D=1;	LCD_DATA_4E=0;	LCD_DATA_4F=1;	LCD_DATA_4G=1;
            break;
        default:
            LCD_DATA_4A=0;	LCD_DATA_4B=0;	LCD_DATA_4C=0;	LCD_DATA_4D=0;	LCD_DATA_4E=0;	LCD_DATA_4F=0;	LCD_DATA_4G=0;
            break;
    }
}
